#uses "fwUnitTestComponentAsserts.ctl" 
#uses "unSystemIntegrity.ctl"

const string TEST_ALARM1 = c_unSystemAlarm_dpPattern + "test1";
const string TEST_ALARM2 = c_unSystemAlarm_dpPattern + "test11"; //make sure to have the name similar to TEST_ALARM1 to catch pattern matching issues

const string TEST_ALARM1_DPE_TO_MATCH = getSystemName()+TEST_ALARM1+".alarm";
const string TEST_ALARM2_DPE_TO_MATCH = getSystemName()+TEST_ALARM2+".alarm";

//------------------------------------------------------------------
void test_unSystemIntegrity_modifyApplicationAlertList_setupSuite(){
  if (dpExists(TEST_ALARM1) == false){
    dpCreate(TEST_ALARM1, c_unSystemAlarm_dpType);
  }
  if (dpExists(TEST_ALARM2) == false){
    dpCreate(TEST_ALARM2, c_unSystemAlarm_dpType);
  }

  assertTrue(dpExists(TEST_ALARM1), TEST_ALARM1 + " could not be created - All tests will fail. "+getLastError());
  assertTrue(dpExists(TEST_ALARM2), TEST_ALARM2 + " could not be created - All tests will fail. "+getLastError());
}
 

//--------------------------------------------------------------------
void test_unSystemIntegrity_modifyApplicationAlertList_addAlarm1(){
  addAlarm(TEST_ALARM1, TEST_ALARM1_DPE_TO_MATCH);
}

void test_unSystemIntegrity_modifyApplicationAlertList_addAlarm2(){
  addAlarm(TEST_ALARM2, TEST_ALARM2_DPE_TO_MATCH);
}

private void addAlarm( string sAlarmDp, string sDpeToMatch){
  dyn_string exceptionInfo;
  dyn_string dp_list;

  _unSystemIntegrity_modifyApplicationAlertList(sAlarmDp, true, exceptionInfo);

  dpGet( UN_SYSTEMINTEGRITY_applicationDPName+":_alert_hdl.._dp_list", dp_list);


  assertEqual(makeDynString(), exceptionInfo, "Error occurred while adding alarm: "+exceptionInfo);
  assertTrue( dynContains(dp_list, sDpeToMatch)>0, "Test alarm "+sAlarmDp+" not added to the application summary alarm");
}

/***
 *  Remove TEST_ALARM1 and make sure that TEST_ALARM2 is still present
 ***/
void test_unSystemIntegrity_modifyApplicationAlertList_removeAlarm1(){
  dyn_string exceptionInfo;
  dyn_string dp_list;

  //first make sure to add the two application to the Application alert list
  addAlarm(TEST_ALARM1, TEST_ALARM1_DPE_TO_MATCH);
  addAlarm(TEST_ALARM2, TEST_ALARM2_DPE_TO_MATCH);

  dpGet( UN_SYSTEMINTEGRITY_applicationDPName+":_alert_hdl.._dp_list", dp_list);
  DebugTN(__FUNCTION__, __LINE__, "dp_List", dp_list);

  //remove TEST_ALARM1
  _unSystemIntegrity_modifyApplicationAlertList(TEST_ALARM1, false, exceptionInfo); 

  dpGet( UN_SYSTEMINTEGRITY_applicationDPName+":_alert_hdl.._dp_list", dp_list);
  DebugTN(__FUNCTION__, __LINE__, "dp_List", dp_list);

  assertEqual(makeDynString(), exceptionInfo, "Error occurred while removing alarm: "+exceptionInfo);
  assertTrue( dynContains(dp_list, TEST_ALARM1_DPE_TO_MATCH)==0, "Test alarm "+TEST_ALARM1+" is still present in the application summary alarm while it should be removed: "+dp_list);
  assertTrue( dynContains(dp_list, TEST_ALARM2_DPE_TO_MATCH)>0, "Test alarm "+TEST_ALARM2+" is not present anymore in the application summary alarm while it should be: "+dp_list);
}

//----------------------------------------------------------------------------
void test_unSystemIntegrity_modifyApplicationAlertList_addNonExistingAlarm(){
  dyn_string exceptionInfo;
  dyn_string dp_list_orig, dp_list_new;

  dpGet( UN_SYSTEMINTEGRITY_applicationDPName+":_alert_hdl.._dp_list", dp_list_orig);

  _unSystemIntegrity_modifyApplicationAlertList("NON_EXISTING_DP", true, exceptionInfo);

  dpGet( UN_SYSTEMINTEGRITY_applicationDPName+":_alert_hdl.._dp_list", dp_list_new);

  assertTrue( dynlen(exceptionInfo)>0, "No error is reported while it should have reported that the non existing dp cannot be added to the summary alarm");
  assertTrue( dp_list_orig==dp_list_new, "List of DP in summary alarm has been modified while it should not have: "+dp_list_orig+" vs "+dp_list_new);
}

//----------------------------------------------------------------------------
void test_unSystemIntegrity_modifyApplicationAlertList_removeNonExistingAlarm(){
  dyn_string exceptionInfo;
  dyn_string dp_list_orig, dp_list_new;

  dpGet( UN_SYSTEMINTEGRITY_applicationDPName+":_alert_hdl.._dp_list", dp_list_orig);

  _unSystemIntegrity_modifyApplicationAlertList("NON_EXISTING_DP", false, exceptionInfo);

  dpGet( UN_SYSTEMINTEGRITY_applicationDPName+":_alert_hdl.._dp_list", dp_list_new);

  assertTrue( dynlen(exceptionInfo)>0, "No error is reported while it should have reported that the non existing dp cannot be added to the summary alarm");
  assertTrue( dp_list_orig==dp_list_new, "List of DP in summary alarm has been modified while it should not have: "+dp_list_orig+" vs "+dp_list_new);
}

//----------------------------------------------------------------------------
void test_unSystemIntegrity_modifyApplicationAlertList_removeAlarmNotYetAdded(){
  dyn_string exceptionInfo;
  dyn_string dp_list_orig, dp_list_new;
  string test_alarm_not_added = "testAlarmNotAdded";

  if (dpExists(test_alarm_not_added) == false){
    dpCreate(test_alarm_not_added, c_unSystemAlarm_dpType);
  }

  dpGet( UN_SYSTEMINTEGRITY_applicationDPName+":_alert_hdl.._dp_list", dp_list_orig);

  _unSystemIntegrity_modifyApplicationAlertList(test_alarm_not_added, false, exceptionInfo);

  dpGet( UN_SYSTEMINTEGRITY_applicationDPName+":_alert_hdl.._dp_list", dp_list_new);

  assertTrue( dynlen(exceptionInfo)==0, "An error is reported while this test should not raise any error: "+exceptionInfo);
  assertTrue( dp_list_orig==dp_list_new, "List of DP in summary alarm has been modified while it should not have: "+dp_list_orig+" vs "+dp_list_new);
}
